from __future__ import print_function, absolute_import, division
from collections import defaultdict
import yamlreader
import logging

from monocyte import Monocyte


def read_config(path):
    return {} if path is None else yamlreader.yaml_load(path)


def convert_arguments_to_config(arguments):
    def parse_csv(csv):
        return [item.strip() for item in csv.split(',')]
    handler_names = parse_csv(arguments["--handler-names"])
    allowed_regions_prefixes = parse_csv(arguments["--allowed-regions-prefixes"])
    ignored_regions = parse_csv(arguments["--ignored-regions"])

    dry_run = (arguments["--dry-run"] != "False")
    ignored_resources = _parse_ignored_resources(arguments["--ignored-resources"])
    config_path = arguments["--config-path"]

    config = {
        "dry_run": dry_run,
        "handler_names": handler_names,
        "allowed_regions_prefixes": allowed_regions_prefixes,
        "ignored_regions": ignored_regions,
        "ignored_resources": ignored_resources,
        "cloudwatchlogs": {}
    }

    cloudwatchlogs_groupname = arguments["--cwl-groupname"]
    if cloudwatchlogs_groupname:
        config["cloudwatchlogs"]["groupname"] = cloudwatchlogs_groupname

    return config_path, config


def apply_default_config(config):
    if config["cloudwatchlogs"]:
        default_config = {
            'region': 'eu-central-1',
            'log_level': 'INFO',
            'groupname': 'monocyte_logs'
        }
        yamlreader.data_merge(default_config, config['cloudwatchlogs'])

        log_level_map = {
            'DEBUG': logging.DEBUG,
            'INFO': logging.INFO,
            'WARN': logging.WARN,
            'ERROR': logging.ERROR
        }
        default_config['log_level'] = log_level_map[default_config['log_level'].upper()]

        config['cloudwatchlogs'] = default_config


def _parse_ignored_resources(ignored_resources):
    parsed = defaultdict(list)
    for resource in ignored_resources.split(","):
        namespace, name = resource.strip().split(".")
        parsed[namespace].append(name)
    return parsed


def main(arguments):
    path, cli_config = convert_arguments_to_config(arguments)
    file_config = read_config(path)
    config = yamlreader.data_merge(file_config, cli_config)
    apply_default_config(config)

    monocyte = Monocyte(**config)

    return monocyte.search_and_destroy_unwanted_resources()



# Monocyte - Search and Destroy unwanted AWS Resources relentlessly.
# Copyright 2015 Immobilien Scout GmbH
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


import boto
from boto.exception import S3ResponseError
from monocyte.handler import Resource, Handler

US_STANDARD_REGION = "us-east-1"


class Bucket(Handler):
    NR_KEYS_TO_SHOW = 4

    def fetch_regions(self):
        return []

    def fetch_unwanted_resources(self):
        for bucket in boto.connect_s3().get_all_buckets():
            try:
                region = bucket.get_location()
            except S3ResponseError as exc:
                # See https://github.com/boto/boto/issues/2741
                if exc.status == 400:
                    self.logger.error(
                        "warning: get_location() crashed for %s, skipping" %
                        bucket.name)
                    continue
                region = "__error__"
            region = region if region else US_STANDARD_REGION
            if self.region_filter(region):
                resource_wrapper = Resource(resource=bucket,
                                            resource_type=self.resource_type,
                                            resource_id=bucket.name,
                                            creation_date=bucket.creation_date,
                                            region=region)
                if bucket.name in self.ignored_resources:
                    self.logger.info('IGNORE ' + self.to_string(resource_wrapper))
                    continue
                yield resource_wrapper

    def to_string(self, resource):
        return "s3 bucket found in {0}, with name {1}, created {2} and {3} entries".format(
            resource.region, resource.wrapped.name,
            resource.wrapped.creation_date,
            len(resource.wrapped.get_all_keys()))

    def delete(self, resource):
        if self.dry_run:
            nr_keys = len(resource.wrapped.get_all_keys())
            if nr_keys:
                keys_log = ["{0}: {1} entries would be removed:".format(resource.wrapped.name, nr_keys)]
                for nr, key in enumerate(resource.wrapped.list()):
                    if nr >= Bucket.NR_KEYS_TO_SHOW:
                        keys_log.append("... ({0} keys omitted)".format(
                            nr_keys - Bucket.NR_KEYS_TO_SHOW))
                        break
                    keys_log.append("'{0}', ".format(key.name))
                self.logger.info("".join(keys_log))
            return
        resource.wrapped.delete_keys(resource.wrapped.list())
        self.logger.info("Initiating deletion sequence for {name}.".format(**vars(resource.wrapped)))

        boto.connect_s3().delete_bucket(resource.wrapped.name)



from __future__ import print_function, absolute_import, division

import boto
import json

from .ses_plugin import AwsSesPlugin


class StatusMailPlugin(AwsSesPlugin):
    def __init__(self, resources, **kwargs):
        self.subject = 'AWS Compliance Checker - Your action is required'
        super(StatusMailPlugin, self).__init__(resources, **kwargs)

    @property
    def body(self):
        email_body = '''Dear AWS User,

our Compliance checker found some AWS resources outside of Europe in your account.
Please check and delete the following resources:

Account: {0}\n'''.format(self._get_account_alias())

        regions = sorted(list(set([res.region for res in self.resources])))
        res_types = sorted(list(set([res.resource_type for res in self.resources])))

        for region in regions:
            region_text = "Region: {0}\n".format(region)
            email_body += region_text
            for res_type in res_types:
                selected_res = (resource for resource in self.resources
                                if resource.region == region and resource.resource_type == res_type)
                for resource in selected_res:
                    res_text = "\t{0} instance with identifier {1}, created {2}\n".format(
                        res_type, resource.resource_id, resource.creation_date)
                    email_body += res_text
        email_footer = '\n Kind regards.\n\tYour Compliance Team'
        email_body += email_footer

        return email_body

    def _get_account_alias(self):
        iam = boto.connect_iam()
        response = iam.get_account_alias()['list_account_aliases_response']
        return response['list_account_aliases_result']['account_aliases'][0]

    def run(self):
        if self.resources:
            self.send_email()


class UsofaStatusMailPlugin(StatusMailPlugin):
    """StatusMailPlugin that finds additional recipients via usofa"""
    def __init__(self, resources, usofa_bucket_name=None, **kwargs):
        super(UsofaStatusMailPlugin, self).__init__(resources, **kwargs)
        self.usofa_bucket_name = usofa_bucket_name

    def _get_usofa_data(self):
        self.conn = boto.s3.connect_to_region(self.region)
        bucket = self.conn.get_bucket(self.usofa_bucket_name)
        key = bucket.get_key('accounts.json')
        account_data = json.loads(key.get_contents_as_string().decode('utf-8'))
        return account_data

    @property
    def recipients(self):
        usofa = self._get_usofa_data()
        account_alias = self._get_account_alias()
        responsible = usofa[account_alias]['email']

        if self.mail_recipients:
            recipients = list(self.mail_recipients)
            recipients.append(responsible)
            return recipients
        return [responsible]
